@@ -50,9 +50,9 @@ lavData <- function(data          = NULL,          # data.frame
                     stop("lavaan WARNING: data argument looks like a covariance matrix; please use the sample.cov argument instead")
                 } else {
                     # or perhaps it is a data matrix?
-                    if(warn) {
-                        warning("lavaan WARNING: data argument has been coerced to a data.frame")
-                    }
+                    #if(warn) {
+                    #    warning("lavaan WARNING: data argument has been coerced to a data.frame")
+                    #}
                     ### FIXME, we should avoid as.data.frame() and handle
                     ### data matrices directly
                     data <- as.data.frame(data, stringsAsFactors = FALSE)
@@ -263,7 +263,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
         # note: by default, we use the order as in the data; 
         # not as in levels(data[,group])
         if(length(group.label) == 0L) {
-            group.label <- unique(as.character(data[,group]))
+            group.label <- unique(as.character(data[[group]]))
             if(warn && any(is.na(group.label))) {
                 warning("lavaan WARNING: group variable ", sQuote(group), 
                         " contains missing values\n", sep="")
@@ -272,7 +272,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
         } else {
             group.label <- unique(as.character(group.label))
             # check if user-provided group labels exist
-            LABEL <- unique(as.character(data[,group]))
+            LABEL <- unique(as.character(data[[group]]))
             idx <- match(group.label, LABEL)
             if(warn && any(is.na(idx))) {
                 warning("lavaan WARNING: some group.labels do not appear ",
@@ -343,7 +343,28 @@ lav_data_full <- function(data          = NULL,          # data.frame
         # does the data contain all the observed variables
         # needed in the user-specified model for this group
         ov.all <- unique(ov.names[[g]], ov.names.x[[g]]) # no overlap if categ
+
+        # handle interactions
+        ov.int.names <- ov.all[ grepl(":", ov.all) ]
+        n.int <- length(ov.int.names)
+        if(n.int > 0L) {
+            ov.names.noint <- ov.all[!ov.all %in% ov.int.names]
+            for(iv in seq_len(n.int)) {
+                NAMES <- strsplit(ov.int.names[iv], ":", fixed = TRUE)[[1L]]
+                if(all(NAMES %in% ov.names.noint)) {
+                    # add this interaction term to the data.frame, unless
+                    # it already exists
+                    if(is.null(data[[ ov.int.names[iv] ]])) {
+                        data[[ ov.int.names[iv] ]] <- 
+                            data[[NAMES[1L]]] * data[[NAMES[2L]]]
+                    }
+                }
+            }
+        }
+
+        # check for missing obsered variables
         idx.missing <- which(!(ov.all %in% names(data)))
+
         if(length(idx.missing)) {
             stop("lavaan ERROR: missing observed variables in dataset: ",
                  paste(ov.all[idx.missing], collapse=" "))
@@ -439,26 +460,26 @@ lav_data_full <- function(data          = NULL,          # data.frame
         # extract cases per group
         if(ngroups > 1L || length(group.label) > 0L) {
             if(missing == "listwise") {
-                case.idx[[g]] <- which(data[, group] == group.label[g] &
-                                       complete.cases(data[,all.idx]))
+                case.idx[[g]] <- which(data[[group]] == group.label[g] &
+                                       complete.cases(data[all.idx]))
                 nobs[[g]] <- length(case.idx[[g]])
-                norig[[g]] <- length(which(data[, group] == group.label[g]))
+                norig[[g]] <- length(which(data[[group]] == group.label[g]))
             } else if(missing == "pairwise" && length(exo.idx) > 0L) {
-                case.idx[[g]] <- which(data[, group] == group.label[g] &
-                                       complete.cases(data[,exo.idx]))
+                case.idx[[g]] <- which(data[[group]] == group.label[g] &
+                                       complete.cases(data[exo.idx]))
                 nobs[[g]] <- length(case.idx[[g]])
-                norig[[g]] <- length(which(data[, group] == group.label[g]))    
+                norig[[g]] <- length(which(data[[group]] == group.label[g]))
             } else {
-                case.idx[[g]] <- which(data[, group] == group.label[g])
+                case.idx[[g]] <- which(data[[group]] == group.label[g])
                 nobs[[g]] <- norig[[g]] <- length(case.idx[[g]])
             }
         } else {
             if(missing == "listwise") {
-                case.idx[[g]] <- which(complete.cases(data[,all.idx]))
+                case.idx[[g]] <- which(complete.cases(data[all.idx]))
                 nobs[[g]] <- length(case.idx[[g]])
                 norig[[g]] <- nrow(data)
             } else if(missing == "pairwise" && length(exo.idx) > 0L) {
-                case.idx[[g]] <- which(complete.cases(data[,exo.idx]))
+                case.idx[[g]] <- which(complete.cases(data[exo.idx]))
                 nobs[[g]] <- length(case.idx[[g]])
                 norig[[g]] <- nrow(data)
             } else {
@@ -509,36 +530,38 @@ lav_data_full <- function(data          = NULL,          # data.frame
         # missing data
         if(missing != "listwise") {
             # get missing patterns
-            Mp[[g]] <- getMissingPatterns(X[[g]])
+            Mp[[g]] <- lav_data_missing_patterns(X[[g]], sort.freq = TRUE,
+                                                 coverage = TRUE)
             # checking!
             if(length(Mp[[g]]$empty.idx) > 0L) {
-                X[[g]] <- X[[g]][-Mp[[g]]$empty.idx,,drop=FALSE]
+                #X[[g]] <- X[[g]][-Mp[[g]]$empty.idx,,drop=FALSE]
                 # remove from case.idx
                 # idx <- which(case.idx[[g]] %in% Mp[[g]]$empty.idx)
-                empty.idx <- Mp[[g]]$empty.idx
-                empty.case.idx <- case.idx[[g]][empty.idx]
-                case.idx[[g]] <- case.idx[[g]][-empty.idx]
+                #empty.idx <- Mp[[g]]$empty.idx
+                #empty.case.idx <- case.idx[[g]][empty.idx]
+                empty.case.idx <- Mp[[g]]$empty.idx
+                #case.idx[[g]] <- case.idx[[g]][-empty.idx]
                 # remove from eXo
-                if(length(exo.idx) > 0L) {
-                    eXo[[g]] <- eXo[[g]][-empty.idx,,drop=FALSE]
-                }
+                #if(length(exo.idx) > 0L) {
+                #    eXo[[g]] <- eXo[[g]][-empty.idx,,drop=FALSE]
+                #}
                 if(warn) {
-                    warning("lavaan WARNING: some cases are empty and will be removed:\n  ", paste(empty.case.idx, collapse=" "))
+                    warning("lavaan WARNING: some cases are empty and will be ignored:\n  ", paste(empty.case.idx, collapse=" "))
                 }
                 # give empty.idx case.idx? (for multiple groups):
-                Mp[[g]]$empty.idx <- empty.case.idx
+                #Mp[[g]]$empty.idx <- empty.case.idx
             }
             if(warn && any(Mp[[g]]$coverage < 0.1)) {
                 warning("lavaan WARNING: due to missing values, some pairwise combinations have less than 10% coverage")
             }
             # in case we had observations with only missings
-            nobs[[g]] <- Mp[[g]]$nobs
+            nobs[[g]] <- NROW(X[[g]]) - length(Mp[[g]]$empty.idx)
         }
 
         # response patterns (categorical only, no exogenous variables)
         all.ordered <- all(ov.names[[g]] %in% ov$name[ov$type == "ordered"])
         if(length(exo.idx) == 0L && all.ordered) {
-            Rp[[g]] <- lav_data_resppatterns(X[[g]])
+            Rp[[g]] <- lav_data_resp_patterns(X[[g]])
         }
 
         # warn if we have a small number of observations (but NO error!)
@@ -575,8 +598,82 @@ lav_data_full <- function(data          = NULL,          # data.frame
     lavData                     
 }
 
-# get response patterns (empty records have already been removed!)
-lav_data_resppatterns <- function(X) {
+# get missing patterns
+lav_data_missing_patterns <- function(Y, sort.freq = FALSE, coverage = FALSE) {
+
+    # construct TRUE/FALSE matrix: TRUE if value is observed
+    OBS <- !is.na(Y)
+
+    # empty cases
+    empty.idx <- which(rowSums(OBS) == 0L)
+
+    # this is what we did in < 0.6
+    #if(length(empty.idx) > 0L) {
+    #    OBS <- OBS[-empty.idx,,drop = FALSE]
+    #}
+
+    # pattern of observed values per observation
+    case.id <- apply(1L * OBS, 1L, paste, collapse = "")
+
+    # remove empty patterns
+    if(length(empty.idx)) {
+        case.id.nonempty <- case.id[-empty.idx]
+    } else {
+        case.id.nonempty <- case.id
+    }
+
+    # sort non-empty patterns (from high occurence to low occurence)
+    if(sort.freq) {
+        TABLE <- sort(table(case.id.nonempty), decreasing = TRUE)
+    } else {
+        TABLE <- table(case.id.nonempty)
+    }
+
+    # unique pattern ids
+    pat.id <- names(TABLE)
+
+    # number of patterns
+    pat.npatterns  <- length(pat.id)
+
+    # case idx per pattern
+    pat.case.idx <- lapply(seq_len(pat.npatterns), 
+                           function(p) which(case.id == pat.id[p]))
+
+    # unique pattern frequencies
+    pat.freq <- as.integer(TABLE)
+
+    # first occurrence of each pattern
+    pat.first <- match(pat.id, case.id)
+
+    # TRUE/FALSE for each pattern
+    pat.obs <- OBS[pat.first,,drop = FALSE] # observed per pattern
+
+    Mp <- list(npatterns = pat.npatterns, id = pat.id, freq = pat.freq,
+               case.idx = pat.case.idx, pat = pat.obs, empty.idx = empty.idx)
+
+    if(coverage) {
+        # FIXME: if we have empty cases, include them in N?
+        # no for now
+        Mp$coverage <- crossprod(OBS) / sum(pat.freq)
+        #Mp$coverage <- crossprod(OBS) / NROW(Y)
+    }
+
+    Mp
+}
+
+# get response patterns (ignore empty cases!)
+lav_data_resp_patterns <- function(X) {
+
+    # construct TRUE/FALSE matrix: TRUE if value is observed
+    OBS <- !is.na(X)
+
+    # empty cases
+    empty.idx <- which(rowSums(OBS) == 0L)
+
+    # remove empty cases
+    if(length(empty.idx) > 0L) {
+        X <- X[-empty.idx,,drop = FALSE]
+    }
 
     ntotal <- nrow(X); nvar <- ncol(X)
 
@@ -590,7 +687,9 @@ lav_data_resppatterns <- function(X) {
     pat <- X[match(order, id), , drop = FALSE]
     row.names(pat) <- as.character(TABLE)
 
-    total.patterns <- prod(apply(X, 2, max))
+    # handle NA?
+    X[is.na(X)] <- -9
+    total.patterns <- prod(apply(X, 2, function(x) length(unique(x))))
     empty.patterns <- total.patterns - npatterns
     # return a list
     #out <- list(nobs=ntotal, nvar=nvar,
