@@ -6,7 +6,6 @@ estimator.ML <- function(Sigma.hat=NULL, Mu.hat=NULL,
 
     # FIXME: WHAT IS THE BEST THING TO DO HERE??
     # CURRENTLY: return Inf  (at least for nlminb, this works well)
-    # used to be: Sigma.hat <- force.pd(Sigma.hat) etc...
     if(!attr(Sigma.hat, "po")) return(Inf)
 
 
@@ -29,6 +28,40 @@ estimator.ML <- function(Sigma.hat=NULL, Mu.hat=NULL,
     fx
 }
 
+# fitting function for standard ML
+estimator.ML_res <- function(Sigma.hat=NULL, Mu.hat=NULL, PI=NULL,
+                             res.cov=NULL, res.int=NULL, res.slopes=NULL,
+                             res.cov.log.det=NULL, 
+                             cov.x = NULL, mean.x = NULL) {
+
+    if(!attr(Sigma.hat, "po")) return(Inf)
+
+    # augmented mean.x + cov.x matrix
+    C3 <- rbind(c(1,mean.x),
+                cbind(mean.x, cov.x + tcrossprod(mean.x)))
+
+    Sigma.hat.inv     <- attr(Sigma.hat, "inv")
+    Sigma.hat.log.det <- attr(Sigma.hat, "log.det")
+    nvar <- ncol(Sigma.hat)
+
+    # sigma
+    objective.sigma <- ( Sigma.hat.log.det + sum(res.cov * Sigma.hat.inv) -
+                         res.cov.log.det - nvar )
+    # beta
+    OBS <- t(cbind(res.int, res.slopes))
+    EST <- t(cbind(Mu.hat, PI))
+    Diff <- OBS - EST
+    objective.beta <- sum(Sigma.hat.inv * crossprod(Diff, C3) %*% Diff)
+
+    fx <- objective.sigma + objective.beta
+
+    # no negative values
+    if(fx < 0.0) fx <- 0.0
+
+    fx
+}
+
+
 # fitting function for restricted ML
 estimator.REML <- function(Sigma.hat=NULL, Mu.hat=NULL,
                            data.cov=NULL, data.mean=NULL,
@@ -131,9 +164,9 @@ estimator.FIML <- function(Sigma.hat=NULL, Mu.hat=NULL, M=NULL, h1=NULL) {
     # for each missing pattern, combine cases and compute raw loglikelihood
     for(p in 1:npatterns) {
 
-        SX <- M[[p]][["SX"]]
-        MX <- M[[p]][["MX"]]
-        w.p[p] <- nobs <- M[[p]][["nobs"]]
+        SX <- M[[p]][["SY"]]
+        MX <- M[[p]][["MY"]]
+        w.p[p] <- nobs <- M[[p]][["freq"]]
         var.idx <- M[[p]][["var.idx"]]
 
         # note: if a decent 'sweep operator' was available (in fortran)
@@ -173,12 +206,16 @@ estimator.FIML <- function(Sigma.hat=NULL, Mu.hat=NULL, M=NULL, h1=NULL) {
 # some changes:
 # - no distinction between x/y (ksi/eta)
 # - loglikelihoods are computed case-wise
-estimator.PML <- function(Sigma.hat = NULL,    # model-based var/cov/cor
-                          TH        = NULL,    # model-based thresholds + means
-                          th.idx    = NULL,    # threshold idx per variable
-                          num.idx   = NULL,    # which variables are numeric
-                          X         = NULL,    # raw data
-                          lavcache     = NULL) {  # housekeeping stuff
+# - 29/03/2016: adapt for exogenous covariates
+estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
+                          TH         = NULL,    # model-based thresholds + means
+                          PI         = NULL,    # slopes
+                          th.idx     = NULL,    # threshold idx per variable
+                          num.idx    = NULL,    # which variables are numeric
+                          X          = NULL,    # raw data
+                          eXo        = NULL,    # eXo data
+                          lavcache   = NULL,    # housekeeping stuff
+                          missing    = NULL) {  # how to deal with missings?
 
     # YR 3 okt 2012
     # the idea is to compute for each pair of variables, the model-based 
@@ -204,6 +241,11 @@ estimator.PML <- function(Sigma.hat = NULL,    # model-based var/cov/cor
     }
 
     nvar <- nrow(Sigma.hat)
+    if(is.null(eXo)) {
+        nexo <- 0L
+    } else {
+        nexo <- NCOL(eXo)
+    }
     pstar <- nvar*(nvar-1)/2
     ov.types <- rep("ordered", nvar)
     if(length(num.idx) > 0L) ov.types[num.idx] <- "numeric"
@@ -216,29 +258,55 @@ estimator.PML <- function(Sigma.hat = NULL,    # model-based var/cov/cor
     PCOL <- col(PSTAR)
 
     # shortcut for all ordered - tablewise
-    if(all(ov.types == "ordered")) {
+    if(all(ov.types == "ordered") && nexo == 0L) {
         # prepare for Myrsini's vectorization scheme
         LONG2 <- LongVecTH.Rho(no.x               = nvar,
                                all.thres          = TH,
                                index.var.of.thres = th.idx, 
                                rho.xixj           = cors)
         # get expected probability per table, per pair
-        PI <- pairwiseExpProbVec(ind.vec = lavcache$LONG, th.rho.vec=LONG2)
+        pairwisePI <- pairwiseExpProbVec(ind.vec = lavcache$LONG, 
+                                         th.rho.vec = LONG2)
         # get frequency per table, per pair
-        logl <- sum(lavcache$bifreq * log(PI))
+        logl <- sum(lavcache$bifreq * log(pairwisePI))
     
         # more convenient fit function
         prop <- lavcache$bifreq / lavcache$nobs
         freq <- lavcache$bifreq
         # remove zero props # FIXME!!! or add 0.5???
-        zero.idx <- which(prop == 0.0)
+        #zero.idx <- which(prop == 0.0)
+        zero.idx <- which( (prop == 0.0) | !is.finite(prop) )
         if(length(zero.idx) > 0L) {
             freq <- freq[-zero.idx]
             prop <- prop[-zero.idx]
-            PI   <- PI[-zero.idx]
+            pairwisePI <- pairwisePI[-zero.idx]
         } 
-        ##Fmin <- sum( prop*log(prop/PI) )
-        Fmin <- sum( freq * log(prop/PI) ) # to avoid 'N'
+        ##Fmin <- sum( prop*log(prop/pairwisePI) )
+        Fmin <- sum( freq * log(prop/pairwisePI) ) # to avoid 'N'
+
+        if(missing == "available.cases") {
+            uniPI <- univariateExpProbVec(TH = TH, th.idx = th.idx)
+
+            # shortcuts
+            unifreq    <- lavcache$unifreq
+            uninobs    <- lavcache$uninobs
+            uniweights <- lavcache$uniweights
+
+            logl <- logl + sum(uniweights * log(uniPI))
+
+            uniprop <- unifreq / uninobs
+
+            # remove zero props 
+            # uni.zero.idx <- which(uniprop == 0.0)
+            uni.zero.idx <- which( (uniprop == 0.0) | !is.finite(uniprop) )
+            if(length(uni.zero.idx) > 0L) {
+                uniprop    <- uniprop[-uni.zero.idx]
+                uniPI      <- uniPI[-uni.zero.idx]
+                uniweights <- uniweights[-uni.zero.idx]
+            }
+
+            Fmin <- Fmin + sum(uniweights * log(uniprop/uniPI))
+        }
 
     } else {
         # # order! first i, then j, lav_matrix_vec(table)!
@@ -246,21 +314,45 @@ estimator.PML <- function(Sigma.hat = NULL,    # model-based var/cov/cor
             for(i in (j+1L):nvar) {
                 pstar.idx <- PSTAR[i,j]
                 # cat("pstar.idx =", pstar.idx, "i = ", i, " j = ", j, "\n")
-                if(ov.types[i] == "numeric" && ov.types[j] == "numeric") {
+                if(ov.types[i] == "numeric" && 
+                   ov.types[j] == "numeric") {
                     # ordinary pearson correlation
                     stop("not done yet")
-                } else if(ov.types[i] == "numeric" && ov.types[j] == "ordered") {
+                } else if(ov.types[i] == "numeric" && 
+                          ov.types[j] == "ordered") {
                     # polyserial correlation
                     stop("not done yet")
-                } else if(ov.types[j] == "numeric" && ov.types[i] == "ordered") {
+                } else if(ov.types[j] == "numeric" && 
+                          ov.types[i] == "ordered") {
                     # polyserial correlation
                     stop("not done yet")
-                } else if(ov.types[i] == "ordered" && ov.types[j] == "ordered") {
+                } else if(ov.types[i] == "ordered" && 
+                          ov.types[j] == "ordered") {
                     # polychoric correlation
-                    PI <- pc_PI(rho   = Sigma.hat[i,j], 
-                                th.y1 = TH[ th.idx == i ],
-                                th.y2 = TH[ th.idx == j ])
-                    LIK[,pstar.idx] <- PI[ cbind(X[,i], X[,j]) ]
+                    if(nexo == 0L) {
+                        pairwisePI <- pc_PI(rho   = Sigma.hat[i,j], 
+                                            th.y1 = TH[ th.idx == i ],
+                                            th.y2 = TH[ th.idx == j ])
+                        LIK[,pstar.idx] <- pairwisePI[ cbind(X[,i], X[,j]) ]
+                    } else {
+                         LIK[,pstar.idx] <- 
+                             pc_lik_PL_with_cov(Y1          = X[,i],
+                                                Y2          = X[,j],
+                                                Rho         = Sigma.hat[i,j],
+                                                th.y1       = TH[th.idx==i],
+                                                th.y2       = TH[th.idx==j],
+                                                eXo         = eXo,
+                                                PI.y1       = PI[i,],
+                                                PI.y2       = PI[j,],
+                                                missing.ind = missing)
+                             #pc_lik2(Y1 = X[, i], Y2 = X[,j], 
+                             #        rho = Sigma.hat[i, j], 
+                             #        th.y1 = TH[th.idx == i], 
+                             #        th.y2 = TH[th.idx == j], 
+                             #        eXo = eXo, 
+                             #        sl.y1 = PI[i, ], 
+                             #        sl.y2 = PI[j, ])
+                    }
                 }
                 #cat("Done\n")
             }
@@ -268,16 +360,55 @@ estimator.PML <- function(Sigma.hat = NULL,    # model-based var/cov/cor
 
         # check for zero likelihoods/probabilities
         # FIXME: or should we replace them with a tiny number?
-        if(any(LIK == 0.0)) return(Inf) # we minimize
+        if(any(LIK == 0.0, na.rm = TRUE)) {
+            OUT <- +Inf
+            attr(OUT, "logl") <- as.numeric(NA)
+            return(OUT)
+        }
  
         # loglikelihood
         LogLIK.cases <- log(LIK)
  
         # sum over cases
-        LogLIK.pairs <- colSums(LogLIK.cases)
+        LogLIK.pairs <- colSums(LogLIK.cases, na.rm = TRUE)
 
         # sum over pairs
-        LogLik <- sum(LogLIK.pairs)
+        LogLik <- logl <- sum(LogLIK.pairs)
+
+        # Fmin
+        Fmin <- (-1)*LogLik
+    }
+
+    if(missing == "available.cases" && all(ov.types == "ordered") && 
+       nexo != 0L) {
+
+       uni_LIK <- matrix(0, nrow(X), ncol(X))
+       for(i in seq_len(nvar)) {
+           uni_LIK[,i] <- uni_lik(Y1    = X[,i],
+                                  th.y1 = TH[th.idx==i],
+                                  eXo   = eXo,
+                                  PI.y1 = PI[i,])
+       }
+
+       if(any(uni_LIK == 0.0, na.rm = TRUE)) {
+           OUT <- +Inf
+           attr(OUT, "logl") <- as.numeric(NA)
+           return(OUT)
+       }
+
+       uni_logLIK_cases <- log(uni_LIK) * lavcache$uniweights.casewise
+
+       #sum over cases
+       uni_logLIK_varwise <- colSums(uni_logLIK_cases)
+
+       #sum over variables
+       uni_logLIK <- sum(uni_logLIK_varwise)
+
+       #add with the pairwise part of LogLik
+       LogLik <- logl <- LogLik + uni_logLIK
+
+       #we minimise
+       Fmin <- (-1)*LogLik
     }
 
     # function value as returned to the minimizer
